local M = {}

local scenario = require("scenario/scenarios")
local helper = require("scenario/scenariohelper")
local player = nil
local playerPos = nil
local playerVel = nil
local playerDistance = nil
local nonPlayers = {}
local racers = {}
local racerPos = nil
local racerVel = nil
local markers = {}
local markerState = 0
local sceneTimer = {} 
local sceneTimerMax = {}
local score = 0
local totalRacers = 0
local isCheater = false
local Racer = {instance = nil, state = true, arrestTimer = 0, arrestDistance = 0, evadeTimer = 0, tracker = nil}

function Racer:new(instance, state, arrestTimer, arrestDistance, evadeTimer, tracker)
	local self = setmetatable({}, Racer)
	self.__index = self
	self.instance = instance or nil
	self.state = state or true
	self.arrestTimer = arrestTimer or 0
	self.arrestDistance = arrestDistance or 0
	self.evadeTimer = evadeTimer or 0
	self.tracker = tracker or nil
	return self
end

local function arrestRacer(theRacer)
	if theRacer.state then
		if theRacer.arrestTimer % 1 == 0 then
			local countdown = 5 - math.modf(theRacer.arrestTimer)
			if countdown > 0 then
				helper.flashUiMessage((tostring(countdown).."..."), 0.5, true)
			end
		end
		theRacer.arrestTimer = theRacer.arrestTimer + 0.25
	else 
		theRacer.arrestTimer = 0
	end 	
	if theRacer.arrestTimer > 5 then
		helper.flashUiMessage("Busted!", 2, true)
		helper.setAiMode(theRacer.instance.name, "stopping")
		theRacer.arrestTimer = 0
		score = score + 1;
		if score >= totalRacers then
			sceneTimerMax[3] = 10000
		end
		theRacer.state = false
	end
end

local function onScenarioUIReady(state)
	core_environment.setTimeOfDay({time=0.16})
	sceneTimer = {0, 0, 0, 0} --Scenario post start timer, marker update timer, scenario post finish timer, extra timer
	sceneTimerMax = {10000, 0, 0, 3}
	score = 0
	totalRacers = 0
	isCheater = false
	player = scenetree.findObject("cta_player0")
	nonPlayers = {scenetree.findObject("cta_ai0"), scenetree.findObject("cta_ai1"), scenetree.findObject("cta_ai2")}
	markers = {scenetree.findObject("cta_marker0"), scenetree.findObject("cta_marker1"), scenetree.findObject("cta_marker2")}
	for i, v in ipairs(nonPlayers) do
		racers[i] = Racer:new(nonPlayers[i], true, 0, 30, 0, markers[i])
		racers[i].tracker.hidden = false
		racers[i].tracker:setScale(Point3F(5, 5, 60))
		racers[i].tracker.instanceColor = ColorF(1, 0, 0, 0.9):asLinear4F()
		helper.setAiMode(racers[i].instance.name, "flee")
		totalRacers = totalRacers + 1
	end
end

local function onRaceTick(raceTickTime)
	for i, v in ipairs(racers) do
		playerPos = player:getPosition()
		racerPos = racers[i].instance:getPosition()
		playerVel = map.objects[map.objectNames[player.name]].vel:length()
		racerVel = map.objects[map.objectNames[racers[i].instance.name]].vel:length()
		playerDistance = (playerPos - racerPos):len()
		if racers[i].state then
			if playerDistance >= 300 then
				helper.setAiAggression(racers[i].instance.name, 1)
			else
				helper.setAiAggression(racers[i].instance.name, 1.15)
			end
			if playerDistance >= 100 then
				markerState = 15
			else
				markerState = 2000
				if racerVel <= 3 then 
					racers[i].arrestDistance = racers[i].arrestDistance + 1 --Arrestable distance grows by 4 per second to prevent rare but unfair situations
				else
					racers[i].arrestDistance = 30
				end
				if sceneTimer[1] >= 5 and playerDistance <= racers[i].arrestDistance and racerVel <= 3 then --More than 5 seconds post start, within distance, within speed
					arrestRacer(racers[i])
				else
					racers[i].arrestTimer = 0
				end
			end
			if sceneTimer[2] >= sceneTimerMax[2] then
				racers[i].tracker:setPosition(vec3(racerPos.x, racerPos.y, racerPos.z + markerState):toPoint3F())
			end	
		end
	end
	if sceneTimer[3] >= 3 then
		scenario.finish({msg = "Money recovered! You are now feared and respected."})
	end
	if isCheater and playerVel <= 6 and sceneTimer[4] >= sceneTimerMax[4] then --Cheater, near idle speeds, and every few seconds; throws a heavy object at the player!
		scenetree.findObject("cta_entity"):setPosition(vec3(playerPos.x, playerPos.y, playerPos.z + 4):toPoint3F())
	end
	if commands.isFreeCamera(0) then
		isCheater = true
	end
	for i, v in ipairs(sceneTimer) do
		if sceneTimer[i] >= sceneTimerMax[i] then
			sceneTimer[i] = 0
		end
		sceneTimer[i] = sceneTimer[i] + 0.25
	end
end

M.onScenarioUIReady = onScenarioUIReady
M.onRaceTick = onRaceTick

return M